#include <velib/base/ve_string.h>
#include <velib/types/variant_print.h>
#include <velib/utils/ve_item_utils.h>

VeVariantUnitFmt veUnitNone = {0, ""};
VeVariantUnitFmt veUnitVolt1Dec = {1, "V"};
VeVariantUnitFmt veUnitVolt2Dec = {2, "V"};
VeVariantUnitFmt veUnitWatt = {0, "W"};
VeVariantUnitFmt veUnitPercentage = {0, "%"};
VeVariantUnitFmt veUnitAmps1Dec = {1, "A"};
VeVariantUnitFmt veUnitCelsius1Dec = {1, "C"};

size_t veItemAlarmText(VeVariant *var, void const *ctx, char *buf, size_t len)
{
	VE_UNUSED(ctx);

	if (veVariantIsValid(var)) {
		switch (var->value.UN32)
		{
		case 0:
			return ve_snprintf(buf, len, "%s", "ok");
		case 1:
			return ve_snprintf(buf, len, "%s", "warning");
		case 2:
			return ve_snprintf(buf, len, "%s", "alarm");
		}
	}

	if (len)
		buf[0] = 0;

	return 0;
}

// Derive power when both valid
void veItemCalculatePower(struct VeItem *V, struct VeItem *I, struct VeItem *P)
{
	VeVariant v;
	VeVariant Voltage;
	VeVariant Current;

	if (!veItemIsValid(V) || !veItemIsValid(I)) {
		veItemInvalidate(P);
		return;
	}

	veItemValue(V, &Voltage); veVariantToFloat(&Voltage);
	veItemValue(I, &Current); veVariantToFloat(&Current);
	veItemOwnerSet(P, veVariantSn32(&v, (sn32) (Voltage.value.Float * Current.value.Float)));
}

static size_t hexString(VeVariant *var, void const *ctx, char *buf, size_t len)
{
	VE_UNUSED(ctx);

	if (!veVariantIsValid(var)) {
		buf[0] = 0;
		return 0;
	}

	return ve_snprintf(buf, len, "%X", var->value.UN32);
}

// Register an already create item
void veItemAddQuantity(struct VeItem *parent, char const *uid, struct VeItem *item, VeVariantUnitFmt *unit)
{
	VeVariant v;

	/*
	 * Startup with an invalid value, so there is always an onchange event
	 * when initial data is obtained. But the path always exists.
	 */
	v.type.tp = VE_UNKNOWN;
	veItemOwnerSet(item, &v);

	/* Explain how to convert it to a textual representation (unit conversion). */
	veItemSetFmt(item, veVariantFmt, unit);

	/* Create the variable to export */
	veItemAddChildByUid(parent, uid, item);
}

void veItemAddBasic(struct VeItem *parent, char const *uid, struct VeItem *item)
{
	veItemAddQuantity(parent, uid, item, &veUnitNone);
}

// Creates / allocates a new item with a unit
struct VeItem *veItemCreateQuantity(struct VeItem *parent, char const *uid, VeVariant *initialValue, VeVariantUnitFmt *unit)
{
	struct VeItem *item;

	item = veItemGetOrCreateUid(parent, uid);
	veItemSetFmt(item, veVariantFmt, unit);
	veItemOwnerSet(item, initialValue);

	return item;
}

struct VeItem *veItemCreateBasic(struct VeItem *parent, char const *uid, VeVariant *initialValue)
{
	return veItemCreateQuantity(parent, uid, initialValue, &veUnitNone);
}

struct VeItem *veItemCreateProductId(struct VeItem *service, un16 productId)
{
	struct VeItem *item;
	VeVariant v;

	item = veItemGetOrCreateUid(service, "ProductId");
	veItemSetFmt(item, hexString, 0);
	veItemOwnerSet(item, veVariantUn32(&v, productId));
	return item;
}
