#include <stdio.h>
#include <string.h>

#include <event2/event.h>
#include <event2/thread.h>

#include <velib/base/base.h>
#include <velib/platform/console.h>
#include <velib/platform/plt.h>
#include <velib/platform/task.h>
#include <velib/utils/ve_logger.h>
#include <velib/utils/ve_timer.h>
#include <velib/utils/ve_todo.h>

#include "platform_init.h"

struct event_base *base;

extern void veDbusUseLibEvent(struct event_base *base);

static void tick(evutil_socket_t fd, short ev, void *ctx)
{
	VE_UNUSED(fd);
	VE_UNUSED(ev);
	VE_UNUSED(ctx);

	taskTick();
	veTodo();
}

/**
 * Libevent can use several event bases nowadays. This function returns
 * a single base.
 */
struct event_base *pltGetLibEventBase(void)
{
	return base;
}

static void updatePersistant(evutil_socket_t fd, short ev, void *ctx)
{
	VE_UNUSED(fd);
	VE_UNUSED(ev);
	VE_UNUSED(ctx);
	taskUpdate();
}

static veBool pltLibEventAddWakeup(evutil_socket_t fd, short flags)
{
	struct event *ev;

	flags |= EV_PERSIST;
	ev = event_new(base, fd, flags, updatePersistant, NULL);
	if (ev == NULL)
		return veFalse;

	event_add(ev, NULL);
	return veTrue;
}

void pltWatchFileDescriptor(int fd)
{
	pltLibEventAddWakeup(fd, EV_READ);
}

int main(int argc, char* argv[])
{
	struct event *ev;
	struct timeval interval;

#if CFG_WITH_PTHREADS
	evthread_use_pthreads();
#endif

	base = event_base_new();
	if (!base) {
		logE("libevent", "could not create event base");
		pltExit(12);
	}

#if CFG_WITH_PTHREADS
	if (evthread_make_base_notifiable(base) < 0) {
		logE("libevent", "Couldn't make base notifiable!");
		pltExit(13);
	}
#endif

#if CFG_WITH_VE_DBUS_ITEM
	veDbusUseLibEvent(base);
#endif

	if ( !(ev = event_new(base, -1, EV_PERSIST, tick, NULL)) ) {
		logE("libevent", "timer creation failed");
		pltExit(14);
	}

	interval.tv_sec = 0;
	interval.tv_usec = 50000;

	if (event_add(ev, &interval) == -1) {
		logE("libevent", "adding timer failed");
		pltExit(15);
	}

	consoleInit(argc, argv);

	event_base_dispatch(base);

	return 0;
}

#if CFG_UTILS_TODO_ENABLED
static struct event *updateEv;

static void update(evutil_socket_t fd, short ev, void *ctx)
{
	VE_UNUSED(fd);
	VE_UNUSED(ev);
	VE_UNUSED(ctx);

	if (updateEv) {
		event_free(updateEv);
		updateEv = NULL;
	}
	taskUpdate();
}

void veTodo(void)
{
	struct timeval interval = { 0 };

	pltInterruptSuspend();

	if (updateEv == NULL) {
		updateEv = evtimer_new(base, update, NULL);
		event_add(updateEv, &interval);
	}

	pltInterruptResume();
}
#endif
